--================================================
--   Created By: Foge
--   Description: UL_Ally_StateBased Behavior
--================================================
--=========================================================
-- UL_Ally_FollowPlayer_DealWithZombie
--=========================================================
local Behavior = CreateAIBehavior("UL_Ally_EngageEnemyMelee", "UL_Ally_BaseBehavior",
{
    Constructor = function(self, entity)
        entity.rangedTargetID = nil;
        entity.primaryTargetID = nil;

        if(not entity.actor.HasContextTag(0, entity.id, "Shooter")) then
            entity.actor.AddContextTag(0, entity.id, "Shooter")
        end

        if(not entity:IsUsingPipe("UL_Ally_Shoot_Target")) then            
            entity:SelectPipe(0,"UL_Ally_DoNothing");
        end
    end,

    Destructor = function(self, entity)
        entity.rangedTargetID = nil

        if(entity.actor.HasContextTag(0, entity.id, "Shooter")) then
            entity.actor.RemoveContextTag(0, entity.id, "Shooter")
        end
        if(entity.actor.HasContextTag(0, entity.id, "Reloading")) then
            entity.actor.RemoveContextTag(0, entity.id, "Reloading")
        end

        entity.primaryTargetID = nil
        if(entity.actor.HasContextTag(0, entity.id, "Behavior_ApproachTarget")) then
            entity.actor.RemoveContextTag(0, entity.id, "Behavior_ApproachTarget");
        end
        if(entity.actor.HasContextTag(0, entity.id, "Behavior_Attack")) then
            entity.actor.RemoveContextTag(0, entity.id, "Behavior_Attack");
        end    

        AI.SetBehaviorVariable(entity.id, "Reloading", false);
        AI.SetBehaviorVariable(entity.id, "DealWithEnemyRanged", false);
        entity.InsertSubpipe("UL_Ally_HoldFire")
    end,


    CREATE_SHOOT_TARGET_PIPE = function(self, entity)
        entity:SelectSecondaryWeapon();
        AI.SetMemoryFireType(entity.id, 2);

        AI.CreateGoalPipe("UL_Ally_Shoot_Target");
            AI.PushGoal("UL_Ally_Shoot_Target", "+ul_locatezombie", 1, entity.rangedTargetID);
            AI.PushGoal("UL_Ally_Shoot_Target", "+branch", 0, "NoTargetFound", IF_LASTOP_FAILED);
            AI.PushGoal("UL_Ally_Shoot_Target", "+acqtarget", 0, "");
            AI.PushGoal("UL_Ally_Shoot_Target", "+firecmd", 1, FIREMODE_AIM, AILASTOPRES_USE, 1.0, -1, AI_FIRECMD_CONTINUE_WHEN_DONE);
            AI.PushGoal("UL_Ally_Shoot_Target", "+firecmd", 0, FIREMODE_BURST, AILASTOPRES_USE);
            AI.PushLabel("UL_Ally_Shoot_Target", "TargetFound");
    --        AI.PushGoal("UL_Ally_Shoot_Target", "+firecmd", 0, FIREMODE_FORCED);
            AI.PushGoal("UL_Ally_Shoot_Target", "+timeout", 1, 0.5);
            AI.PushGoal("UL_Ally_Shoot_Target", "+ul_locatezombie", 1, entity.rangedTargetID);
            AI.PushGoal("UL_Ally_Shoot_Target", "+branch", 0, "TargetFound", IF_LASTOP_FAILED);
            AI.PushGoal("UL_Ally_Shoot_Target", "+acqtarget", 0, "");
            AI.PushGoal("UL_Ally_Shoot_Target", "+branch", 0, "TargetFound", BRANCH_ALWAYS);
            AI.PushLabel("UL_Ally_Shoot_Target", "NoTargetFound");
            AI.PushGoal("UL_Ally_Shoot_Target", "+signal", 1, 1, "TARGETING_FAILED", SIGNALFILTER_SENDER);
            AI.PushGoal("UL_Ally_Shoot_Target", "+firecmd", 0, FIREMODE_OFF);
            AI.PushLabel("UL_Ally_Shoot_Target", "END");
        entity:SelectPipe(0, 'UL_Ally_Shoot_Target');
    end,
    
    CREATE_APPROACH_TARGET_PIPE = function(self, entity)

        AI.CreateGoalPipe("UL_Ally_Approach_Primary_Target");
--            AI.PushGoal("UL_Ally_Approach_Primary_Target", "+run", 0, 2);
            AI.PushGoal("UL_Ally_Approach_Primary_Target", "+ul_locatezombie", 1, entity.primaryTargetID);
            AI.PushGoal("UL_Ally_Approach_Primary_Target", "+branch", 1, "TRY_AGAIN", IF_LASTOP_FAILED);
            AI.PushGoal("UL_Ally_Approach_Primary_Target", "+stick", 1, 1.0, AILASTOPRES_USE+AI_REQUEST_PARTIAL_PATH, STICK_BREAK+STICK_SHORTCUTNAV);
            AI.PushGoal("UL_Ally_Approach_Primary_Target", "+branch", 1, "END", IF_LASTOP_SUCCEED);
            AI.PushLabel("UL_Ally_Approach_Primary_Target", "TRY_AGAIN");
            AI.PushGoal("UL_Ally_Approach_Primary_Target", 'signal', 1, 1, 'TRY_AGAIN', SIGNALFILTER_SENDER)
            AI.PushLabel("UL_Ally_Approach_Primary_Target", "END");

        if(not entity.actor.HasContextTag(0, entity.id, "Behavior_ApproachTarget")) then
            entity.actor.AddContextTag(0, entity.id, "Behavior_ApproachTarget");
        end

        
        entity:SelectPipe(0,"UL_Ally_Approach_Primary_Target");
    end,
    
    TRY_AGAIN = function(self, entity)
        Log("ENGAGE MELEE ERROR -- "..EntityName(entity.id).." has an invalid target or a pathfinding failure!!!");

        entity:SelectPipe(0,"do_nothing");     
        entity.primaryTargetID = entity.actor:GetEntitySlot(2);

        if(entity.primaryTargetID ~= 0) then
            entity.Behavior:CREATE_APPROACH_TARGET_PIPE(entity)
        end
    end, 

    NEW_SHOOT_TARGET = function(self, entity)
        entity.rangedTargetID = entity.actor:GetEntitySlot(22)
        if(entity.rangedTargetID == nil) then
            entity.rangedTargetID = entity.actor:GetEntitySlot(2);
        end

        if(entity.rangedTargetID == nil) then
            Log(EntityName(entity.id).." has no valid ranged target!!!");
            entity.primaryTargetID = entity.actor:GetEntitySlot(2);

            if(entity.primaryTargetID ~= 0) then
           	entity.Behavior:CREATE_APPROACH_TARGET_PIPE(entity)
            end
        else
            local targetEntity = System.GetEntity(entity.rangedTargetID)

            if(targetEntity.class == "SwatZombie" or targetEntity.class == "ArmyZombie") then
                Log(EntityName(entity.id).."'s target is ARMORED.");
                entity.primaryTargetID = entity.actor:GetEntitySlot(2);

                if(entity.primaryTargetID ~= 0) then
            		entity.Behavior:CREATE_APPROACH_TARGET_PIPE(entity)
                end
            else
        	entity:SelectPipe(0, "UL_Ally_DoNothing");
                entity.Behavior:CREATE_SHOOT_TARGET_PIPE(entity);
            end
        end
    end,

    TARGETING_FAILED = function(self, entity)
        Log("UL_Ally_Shoot_Target -- "..EntityName(entity.id).." has failed to locate target: "..EntityName(entity.rangedTargetID).." !!!!");
        entity.Behavior:NEW_SHOOT_TARGET(entity);
    end,
})
