--=========================================================
-- UL_Ally_FollowPlayer_Vehicle
--=========================================================

local Behavior = CreateAIBehavior("UL_Ally_FollowPlayer_Vehicle", "UL_Ally_BaseBehavior",
{
    Constructor = function(self, entity)
        --Log(entity:GetName().."UL_Ally_FollowPlayer_Vehicle Constructor");
        entity.vehicle = nil

        if(not entity.actor.HasContextTag(0, entity.id, "AllowVehicle")) then
            entity.actor.AddContextTag(0, entity.id, "AllowVehicle");
        end

        if(not entity.actor.HasContextTag(0, entity.id, "Dodge")) then
            entity.actor.AddContextTag(0, entity.id, "Dodge");
        end

        entity:SelectPipe(0, "UL_Ally_DoNothing");

        self:GetInVehicle(entity)
    end,

    Destructor = function(self, entity)
        --Log(entity:GetName().."UL_Ally_FollowPlayer_Vehicle Destructor");
        if(not AI.GetBehaviorVariable(entity.id, "InVehicle")) then
            local vehicle = entity.AI.theVehicle
            if vehicle then
                vehicle.vehicle:ExitVehicle(entity.id)
                entity.AI.theVehicle = nil
                entity.AI.mySeat = nil
            end
            if entity:IsUsingPipe("UL_Ally_EnterVehicle") then
                entity:CancelSubpipe();
            end
            if(entity.actor.HasContextTag(0, entity.id, "AllowVehicle")) then
                entity.actor.RemoveContextTag(0, entity.id, "AllowVehicle");
            end
        end

        if(entity.actor.HasContextTag(0, entity.id, "Dodge")) then
            entity.actor.RemoveContextTag(0, entity.id, "Dodge");
        end

        if entity.AI.GetInVehicleTimer then
            Script.KillTimer(entity.AI.GetInVehicleTimer)
            entity.AI.GetInVehicleTimer = nil
        end
    end,

    GetInVehicleTimerCallback = function(dataTable)
        local self = dataTable.self
        local entity = dataTable.entity
        entity.AI.GetInVehicleTimer = nil
        self:GetInVehicle(entity)
    end,

    GetInVehicle = function(self, entity)
        local didEnter = false
        local vehicle = g_localActor.AI.theVehicle
        if vehicle then
            local usable, seatId = VehicleManager.IsVehicleUsable(vehicle.id, entity.id)
            if usable then
                didEnter = VehicleManager.EnterVehicle(vehicle.id, entity.id, seatId)
                if didEnter then
                    --Log(EntityName(entity)..': EnterVehicle: '..tostring(didEnter))
                    entity.AI.theVehicle = vehicle
                    entity.AI.mySeat = seatId
                    if entity:IsUsingPipe("UL_Ally_EnterVehicle") then
                        entity:CancelSubpipe()
                    end
                    entity:InsertSubpipe(0, "UL_Ally_EnterVehicle")
                else
                    --Log(EntityName(entity)..': failed to enter vehicle')
                end
            else
                --Log(EntityName(entity)..': vehicle is not usable')
            end
        else
            --Log(EntityName(entity)..': player does not have a vehicle')
        end

        if not didEnter then
            if not entity.AI.GetInVehicleTimer then
                entity.AI.GetInVehicleTimer = Script.SetTimerForFunction(100, "AIBehavior.UL_Ally_FollowPlayer_Vehicle.GetInVehicleTimerCallback", { self = self, entity = entity })
            end
        end
    end,
})

local Behavior = CreateAIBehavior("UL_Ally_Vehicle", "UL_Ally_BaseBehavior",
{
    Constructor = function(self, entity)
        entity:CancelSubpipe();
    end,

    Destructor = function(self, entity)
    end,
    
    ModeMoveTo = function(self, entity, sender, data)
        --Log(EntityName(entity.id).." ModeMoveTo");
    
        entity.moveToData = data;

        local dataTable = {selfRef = self, entityRef = entity};
        UL_Ally_Utils:RangedAttacker_Cleanup(dataTable)
       
        entity.nextBehavior = self.BEHAVIOR_MOVE_TO
        AI.SetBehaviorVariable(entity.id, 'ExitBehavior', true)
        
        if(entity.actor.HasContextTag(0, entity.id, "AllowVehicle")) then
            entity.actor.RemoveContextTag(0, entity.id, "AllowVehicle");
        end
    end,    
})